# 安卓项目辅助工具

## 工具列表
1. 删除没有被引用到的图片/xml文件
2. 将图片/xml文件移动到使用的模块
3. 将string.xml的字符串进行删除或者移动到对应模块

---

## 工具的使用方式
  * 在项目中️以模块的形式导入:
  * 项目中根目录找到setting.gradle,加入刚刚复制的文件夹: include ':RemoveUnusedImg'  
  * 找到需要使用工具的入口,然后修改部分Java代码,以Java的形式运行即可

---

## 1. 删除没有被引用到的图片/xml文件
### 使用场景:
  AndroidStudio自带的移除未使用资源有bug,有时候明明有在使用确被删掉了,所以用此工具作为替代
### 入口:
UnuseRes.kt的main方法
### 原理:
  1. 遍历指定目录,读取出所有的切图文件,例如png jpg webp.......
  2. 遍历出所有的代码文件 *.java *.kt *.xml
  3. 因为在代码中有使用到资源的代码,都是不会中间换行的 ,
  所以遍历上一步所有代码文件,逐行检查每一行代码,看看是否有使用调用到图片素材的关键字,例如"@drawable" "@mipmap" 等,将所有有使用该行代码一整行存起来
  4. 在第3步结果中,每个结果都跟第一步得到的全部图片进行匹配,如果有匹配到则标记为该资源已被使用
  5. 最后再从第1步所有切图文件 , 去除第4步的结果 , 得到未被使用的资源 , 删除

  * 对xml类型的图片资源原理也是类似的

### 使用方法
* 首先指定要检查的目录 , 打开UnuseRes.kt ,修改要检查的模块目录,直接填写绝对路径也行,之后会对该目录递归查找
* 然后运行UnuseRes.kt的main方法
* 当有xml使用了一些切图,但是并没有代码使用该xml文件,会导致该文件显示为已标记,无法删除  所以应删除xml之后再运行一次,才会删除图片. 建议多运行几次,结果都为0之后才算完成
* **为了防止运行的时候误删除,默认注释掉删除文件的功能,运行的时候只会打印日志,第一次使用应先检查日志打印的内容,然后再用AS随机查找一下资源是否会被使用.
随机检查几项后 , 确认没被使用后去除注释再运行 , 之后将会删除所有未被使用的资源**

* **删除操作不可逆 , 确保在有Git的仓库上进行操作 , 或者有备份的情况下进行操作**
* **删除操作不可逆 , 确保在有Git的仓库上进行操作 , 或者有备份的情况下进行操作**
* **删除操作不可逆 , 确保在有Git的仓库上进行操作 , 或者有备份的情况下进行操作**

---

## 2. 将图片/xml文件移动到使用的模块
### 使用场景:
  在组件化的项目中,以APP模块为例,一般会依赖base模块,APP模块使用的部分图片或者xml资源放在base模块中,这时候一一把APP模块使用的资源从base模块找出来,并且移入APP模块下,可使用此工具进行分析
### 入口:
SingleImplement.kt的main方法
### 原理:
跟第一个工具差不多,前三步跟第一个工具一样

4. 统计此文件被哪些代码文件依赖,存到列表中

5. 对每个资源文件的依赖列表进行判断,通过路径得到所属的模块,再通过set进行去重.

6. 第五步得到的结果数量为1的说明就是被一个模块依赖了,再通过代码执行git命令,将文件移动过去
### 使用方法
跟第一个工具的使用方法差不多,图片和xml都是分开来操作的.  所以需要循环操作几次,均为0的时候才算完全完成
*  ***请在有GIT跟踪的目录下进行***

-------------------------

## 3. 将string.xml的字符串进行删除或者移动到对应模块
### 使用场景:
此代码跟第二个工具的作用差不多,只不过处理的对象是每个模块下的string.xml , 需要清除一些模块下string.xml文件没有用到的string内容 , 或者是将其内容移动到引用代码所对应的模块下 , 可以使用此工具来完成
### 入口
StringPull.kt的main方法
### 原理:
1. 给定一个路径,递归扫描该路径下的所有*.xml文件 , 并且判断是不是字符串的文件
2. 对扫描到的string.xml文件 , 获取所有定义的参数以及数值
3. 扫描所有代码
4. 对第一步的结果,进行判断,得到每个字符串引用的次数
5. 引用次数为0则解析xml , 并且删除该字符串内容
6. 引用次数为1的字符串 , 则视为只被一个模块调用 , 如果调用的模块不是本模块 , 则移动到对应的模块去

### 使用方法
* 找到主方法 , 检查realExecuteCmd是否为false , false为调试模式
* 主方法配置的核对路径是否为需要扫描的路径
* 调试模式下运行主方法 , 之后会打印日志
* 根据打印的日志 , 随机挑选几条数据进行人工校验 , 没问题之后直接调用主方法

### 注意事项
* 当某些模块的string.xml不存在时会自动创建 , 随后会用命令行添加到git中 , 由于使用的是"git add ." 可能会导致添加了一些不需要的文件 , 在提交的时候应该检查一下.如果有不需要add的可以使用 "git rm --cached xxx" 命令来移除添加的文件
* 由于有些字符串可能是在SDK调用的,这个工具时候并不一定能生效 , 可能会把有些有用的字符串给删除掉 , 这种情况可以添加到白名单解决 , 自定查找 isWhiteList 方法,进行手工添加
